package com.qmth.sop.business.service;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.service.IService;
import com.qmth.sop.business.bean.params.FlowApproveParam;
import com.qmth.sop.business.bean.params.SopApplyParam;
import com.qmth.sop.business.bean.params.SopInfoListParam;
import com.qmth.sop.business.bean.params.SopPublishParam;
import com.qmth.sop.business.bean.result.FlowTaskResult;
import com.qmth.sop.business.entity.*;
import com.qmth.sop.common.enums.FlowApprovePassEnum;

import java.util.List;
import java.util.Map;

/**
 * <p>
 * sop表 服务类
 * </p>
 *
 * @author wangliang
 * @since 2023-08-01
 */
public interface TBSopInfoService extends IService<TBSopInfo> {

    /**
     * 查询动态sop表名是否存在
     *
     * @param db
     * @param tableName
     * @return
     */
    int selectSopDynamicExist(String db, String tableName);

    /**
     * 保存动态sop信息
     *
     * @param tfCustomFlow
     * @param code
     * @param flowId
     * @param flowTaskResult
     * @return
     */
    Boolean saveDynamicSop(TFCustomFlow tfCustomFlow, String code, Long flowId, FlowTaskResult flowTaskResult);

    /**
     * 查询动态sop数据
     *
     * @param tableName
     * @param flowDeploymentId
     * @param flowId
     * @return
     */
    int selectDynamicSop(String tableName, String flowDeploymentId, Long flowId);

    /**
     * sop发布信息
     *
     * @param sopPublishParam
     * @return
     */
    Boolean sopPublish(SopPublishParam sopPublishParam) throws InterruptedException;

    /**
     * sop申请信息
     *
     * @param sopApplyParam
     * @return
     */
    Boolean sopApply(SopApplyParam sopApplyParam) throws InterruptedException;

    /**
     * sop填报信息
     *
     * @param flowApproveParam
     * @return
     */
    Boolean sopApprove(FlowApproveParam flowApproveParam) throws InterruptedException;

    /**
     * sop管理列表
     *
     * @param sopInfoListParam
     * @return
     */
    IPage<Map> list(SopInfoListParam sopInfoListParam);

    /**
     * sop申请或发布
     *
     * @param sopNo
     * @param crmNo
     * @param flowDeploymentId
     * @param approve
     * @param formProperties
     * @return
     * @throws InterruptedException
     */
    public Boolean sopApplyOrPublish(String sopNo, String crmNo, String flowDeploymentId, FlowApprovePassEnum approve, String formProperties) throws InterruptedException;

    /**
     * 根据crmNo查询在途的sop信息
     *
     * @param crmNo
     * @return
     */
    List<String> findFlowPassageByCrmNo(String crmNo);

    /**
     * sop自动审批
     *
     * @param tfCustomFlow
     * @param tfCustomFlowEntity
     * @param tfFlowApprove
     * @param tbSopInfoDetail
     * @param crmNo
     */
    public void sopSystemApprove(TFCustomFlow tfCustomFlow, TFCustomFlowEntity tfCustomFlowEntity, TFFlowApprove tfFlowApprove, TBSopInfoDetail tbSopInfoDetail, String crmNo) throws InterruptedException;

    /**
     * 保存提醒/延时任务
     *
     * @param map
     */
    public void saveJobRemind(Map<String, Object> map);
}
