package com.qmth.sop.business.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.qmth.sop.business.activiti.service.ActivitiService;
import com.qmth.sop.business.bean.dto.UserDto;
import com.qmth.sop.business.bean.params.FlowApproveParam;
import com.qmth.sop.business.bean.result.ProjectExchangeResult;
import com.qmth.sop.business.entity.SysUser;
import com.qmth.sop.business.entity.TBProjectExchange;
import com.qmth.sop.business.entity.TFCustomFlowEntity;
import com.qmth.sop.business.mapper.TBProjectExchangeMapper;
import com.qmth.sop.business.service.SysUserRoleService;
import com.qmth.sop.business.service.TBProjectExchangeService;
import com.qmth.sop.business.service.TFCustomFlowEntityService;
import com.qmth.sop.common.contant.SystemConstant;
import com.qmth.sop.common.enums.*;
import com.qmth.sop.common.util.ServletUtil;
import org.activiti.engine.TaskService;
import org.activiti.engine.task.Task;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.CollectionUtils;

import javax.annotation.Resource;
import java.util.*;
import java.util.stream.Collectors;

/**
 * <p>
 * 项目变更计划表 服务实现类
 * </p>
 *
 * @author wangliang
 * @since 2023-08-01
 */
@Service
public class TBProjectExchangeServiceImpl extends ServiceImpl<TBProjectExchangeMapper, TBProjectExchange> implements TBProjectExchangeService {

    @Resource
    ActivitiService activitiService;

    @Resource
    TFCustomFlowEntityService tfCustomFlowEntityService;

    @Resource
    TaskService taskService;

    @Resource
    TBProjectExchangeService tbProjectExchangeService;

    @Resource
    SysUserRoleService sysUserRoleService;

    /**
     * 新增项目变更计划申请
     *
     * @param tbProjectExchange
     * @return
     * @throws InterruptedException
     */
    @Override
    @Transactional
    public Boolean saveProjectExchange(TBProjectExchange tbProjectExchange) throws InterruptedException {
        if (tbProjectExchange.getFlowApprove() != FlowApprovePassEnum.START && tbProjectExchange.getFlowApprove() != FlowApprovePassEnum.DRAFT) {
            throw ExceptionResultEnum.PARAMS_ERROR.exception("项目计划变更申请只能为提交或草稿");
        }
        SysUser sysUser = (SysUser) ServletUtil.getRequestUser();
        //查询所有QA角色
        List<UserDto> userDtoList = sysUserRoleService.userListByRoleType(RoleTypeEnum.QA);
        if (CollectionUtils.isEmpty(userDtoList)) {
            throw ExceptionResultEnum.ERROR.exception("系统中没有QA角色的用户");
        }
        List<String> approveUserIds = userDtoList.stream().map(s -> String.valueOf(s.getId())).collect(Collectors.toList());
        FlowApproveParam flowApproveParam = new FlowApproveParam(tbProjectExchange.getFlowDeploymentId(), tbProjectExchange.getFlowApprove(), approveUserIds, tbProjectExchange.getCrmNo());
        flowApproveParam.setApproveRemark(tbProjectExchange.getFlowApprove().getTitle());
        Map<String, Object> map = activitiService.taskApprove(flowApproveParam);
        TFCustomFlowEntity tfCustomFlowEntity = (TFCustomFlowEntity) map.get(SystemConstant.FLOW_ENTITY);
        tbProjectExchange.setCode(tfCustomFlowEntity.getCode(), sysUser.getId());
        tfCustomFlowEntity.setObjId(tbProjectExchange.getId());
        tfCustomFlowEntityService.updateById(tfCustomFlowEntity);
        return tbProjectExchangeService.save(tbProjectExchange);
    }

    /**
     * 项目变更计划申请列表
     *
     * @param iPage
     * @param serviceId
     * @param createUserId
     * @param type
     * @param customType
     * @param flowStatus
     * @param customName
     * @param exchangeNo
     * @param startTime
     * @param endTime
     * @return
     */
    @Override
    public IPage<ProjectExchangeResult> list(IPage<Map> iPage, Long serviceId, Long createUserId, DelayWarnTypeEnum type, ProductTypeEnum customType, ProjectExchangeFlowStatusEnum flowStatus, String customName, String exchangeNo, Long startTime, Long endTime) {
        SysUser sysUser = (SysUser) ServletUtil.getRequestUser();
        createUserId = Objects.isNull(createUserId) ? sysUser.getId() : createUserId;
        return this.baseMapper.list(iPage, serviceId, createUserId, Objects.nonNull(type) ? type.name() : null, Objects.nonNull(customType) ? customType.name() : null, Objects.nonNull(flowStatus) ? flowStatus.name() : null, customName, exchangeNo, startTime, endTime);
    }

    /**
     * 项目计划变更审核接口
     *
     * @param taskId
     * @param flowApprove
     * @param projectExchangeApprove
     * @param remark
     * @param userId
     * @return
     * @throws InterruptedException
     */
    @Override
    @Transactional
    public Boolean flowApprove(Long taskId, FlowApprovePassEnum flowApprove, ProjectExchangeApproveEnum projectExchangeApprove, String remark, Long userId) throws InterruptedException {
        Optional.ofNullable(taskId).orElseThrow(() -> ExceptionResultEnum.FLOW_TASK_ID_IS_NULL.exception());
        if (flowApprove != FlowApprovePassEnum.PASS && flowApprove != FlowApprovePassEnum.DRAFT) {
            throw ExceptionResultEnum.PARAMS_ERROR.exception("项目计划变更审批只能为通过或草稿");
        }
        SysUser sysUser = (SysUser) ServletUtil.getRequestUser();
        userId = Objects.nonNull(sysUser) ? sysUser.getId() : userId;
        Task task = taskService.createTaskQuery().taskId(String.valueOf(taskId)).singleResult();
        Optional.ofNullable(task).orElseThrow(() -> ExceptionResultEnum.FLOW_TASK_NO_DATA.exception());

        TFCustomFlowEntity tfCustomFlowEntity = tfCustomFlowEntityService.getOne(new QueryWrapper<TFCustomFlowEntity>().lambda().eq(TFCustomFlowEntity::getFlowId, Long.parseLong(task.getProcessInstanceId())));
        activitiService.taskApprove(new FlowApproveParam(taskId, flowApprove, Arrays.asList(String.valueOf(userId)), tfCustomFlowEntity.getCrmNo(), projectExchangeApprove.getTitle() + ";" + remark));

        TBProjectExchange tbProjectExchange = this.getOne(new QueryWrapper<TBProjectExchange>().lambda().eq(TBProjectExchange::getExchangeNo, tfCustomFlowEntity.getCode()));
        Optional.ofNullable(tbProjectExchange).orElseThrow(() -> ExceptionResultEnum.PROJECT_EXCHANGE_NO_DATA.exception());

        tbProjectExchange.setResult(projectExchangeApprove);
        tbProjectExchange.setRemark(remark);
        return tbProjectExchangeService.updateById(tbProjectExchange);
    }

    /**
     * 项目计划变更详情接口
     *
     * @param id
     * @return
     */
    @Override
    public Object view(Long id) {
        TBProjectExchange tbProjectExchange = this.getById(id);
        Optional.ofNullable(tbProjectExchange).orElseThrow(() -> ExceptionResultEnum.PROJECT_EXCHANGE_NO_DATA.exception());

        //TODO 需要根据crm单号取crm相关信息
        return tbProjectExchange;
    }
}
